#!/usr/bin/env python
#
# Setup script for sk1sdk 
#
# Copyright (c) 2010 Igor E. Novikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
#
# Usage: 
# --------------------------------------------------------------------------
#  to build package:   python setup.py build
# --------------------------------------------------------------------------
#  to install package:   python setup.py install
# --------------------------------------------------------------------------
#  to create source distribution:   python setup.py sdist
# --------------------------------------------------------------------------
#  to create binary RPM distribution:  python setup.py bdist_rpm
# --------------------------------------------------------------------------
#  to create binary DEB package:  python setup.py bdist_deb
# --------------------------------------------------------------------------
#  help on available distribution formats: python setup.py bdist --help-formats
#


import os, sys

COPY=False
DEBIAN=False
VERSION='0.9.1pre2'


#Return full  directory names list for provided path	
def get_dirs_withpath(path='.'):
	list=[]
	names=[]
	if os.path.isdir(path):
		try:
			names = os.listdir(path)
		except os.error:
			return names
	names.sort()
	for name in names:
		if os.path.isdir(os.path.join(path, name)) and not name=='.svn':
			list.append(os.path.join(path, name))
	return list

#Return recursive directories list for provided path
def get_dirs_tree(path='.'):
	tree=get_dirs_withpath(path)
	res=[]+tree
	for node in tree:
		subtree=get_dirs_tree(node)
		res+=subtree
	return res	

if __name__ == "__main__":
	
	print 'Source tree scan...'
	dirs=get_dirs_tree('src/tkstyle')
	
	share_dirs=[]
	for item in dirs:
		share_dirs.append(os.path.join(item[4:],'*.*'))
	
	if len(sys.argv)>1 and sys.argv[1]=='build&copy':
		COPY=True
		sys.argv[1]='build'

	if len(sys.argv)>1 and sys.argv[1]=='bdist_deb':
		DEBIAN=True
		sys.argv[1]='build'
			
	from distutils.core import setup, Extension

	for item in ['GNU_LGPL_v2', 'COPYRIGHTS']:
		share_dirs.append(item)
		
		
	#Fix for Debian based distros
	tcl_include_dirs = []
	tcl_ver=''
	if os.path.isdir('/usr/include/tcl8.5'):
		tcl_include_dirs = ['/usr/include/tcl8.5']
		tcl_ver ='8.5'
		
	if os.path.isdir('/usr/include/tcl8.6'):
		tcl_include_dirs = ['/usr/include/tcl8.6']
		tcl_ver ='8.6'
	
	#Fix for OpenSuse
	if not tcl_ver:
		if os.path.isfile('/usr/lib/libtcl8.5.so'):
			tcl_ver ='8.5'
		if os.path.isfile('/usr/lib/libtcl8.6.so'):
			tcl_ver ='8.6'
	
	#Fix for Fedora
	if not tcl_ver:
		if os.path.isfile('/usr/lib64/libtcl8.5.so'):
			tcl_ver ='8.5'
		if os.path.isfile('/usr/lib64/libtcl8.6.so'):
			tcl_ver ='8.6'
			
	if not tcl_ver:
		print 'System tcl/tk =>8.5 libraries have not found!'
		sys.exit(1)
 
		
 	tkimage_src='src/tkimage/'			
	tkimage_module = Extension('sk1sdk.tkimage._imagingtk',
			define_macros = [('MAJOR_VERSION', '1'),
						('MINOR_VERSION', '0')],
			sources = [tkimage_src+'_imagingtk.c', tkimage_src+'tkImaging.c'],
			include_dirs = tcl_include_dirs,
			libraries=['tk'+tcl_ver, 'tcl'+tcl_ver],
			extra_compile_args=["-Wall"])
	
 	tkpng_src='src/tkpng/libtkpng/'
	tkpng_module = Extension('sk1sdk.tkpng.libtkpng',
			define_macros = [('MAJOR_VERSION', '0'),
						('MINOR_VERSION', '9')],
			sources = [tkpng_src+'tkImgPNG.c', tkpng_src+'tkImgPNGInit.c'],
			include_dirs = tcl_include_dirs,
			libraries=['tk'+tcl_ver, 'tcl'+tcl_ver, 'z'])
	
 	tkXcursor_src='src/tkXcursor/'
 	tkXcursor_include_dirs=['/usr/include/X11/Xcursor']
 	tkXcursor_include_dirs.extend(tcl_include_dirs)
	tkXcursor_module = Extension('sk1sdk.tkXcursor._tkXcursor',
			define_macros = [('MAJOR_VERSION', '1'),
						('MINOR_VERSION', '0')],
			sources = [tkXcursor_src+'_tkXcursor.c'],
			include_dirs=tkXcursor_include_dirs,
			libraries=['tk'+tcl_ver, 'tcl'+tcl_ver, 'Xcursor'])			
			
 	paxtkinter_src='src/libtk/libtkinter/'				
	paxtkinter_module = Extension('sk1sdk.libtk._tkinter',
			define_macros = [('MAJOR_VERSION', '1'),
						('MINOR_VERSION', '0')],
			sources = [paxtkinter_src+'_tkinter.c'],
			include_dirs = tcl_include_dirs,
			libraries=['tk'+tcl_ver, 'tcl'+tcl_ver])
			
	setup (name = 'sk1sdk',
			version = VERSION,
			description = 'sk1sdk is a set of python non-GUI extensions for sK1 Project',
			author = 'Igor E. Novikov',
			author_email = 'igor.e.novikov@gmail.com',
			maintainer = 'Igor E. Novikov',
			maintainer_email = 'igor.e.novikov@gmail.com',
			license = 'LGPL v2',
			url = 'http://sk1project.org',
			download_url = 'http://sk1project.org/',
			long_description = '''sk1sdk is a multiplatform development kit for sK1 and related applications. sK1 Team (http://sk1project.org), copyright (c) 2010 by Igor E. Novikov.
			''',
		classifiers=[
			'Development Status :: 5 - Stable',
			'Environment :: Desktop',
			'Environment :: Console',
			'Environment :: Web Environment',
			'Intended Audience :: End Users/Desktop',
			'Intended Audience :: Developers',
			'License :: OSI Approved :: LGPL v2',
			'Operating System :: POSIX',
			'Operating System :: Microsoft :: Windows',
			'Operating System :: MacOS :: MacOS X',
			'Programming Language :: Python',
			'Programming Language :: C',
			"Topic :: Multimedia :: Graphics",
			],

		packages = ['sk1sdk',
				'sk1sdk.tkimage',
				'sk1sdk.libtk',
				'sk1sdk.libttk',
				'sk1sdk.tkpng',
				'sk1sdk.tkstyle',
				'sk1sdk.tkXcursor',
				],

			
		package_dir = {'sk1sdk': 'src',
		'sk1sdk.tkimage': 'src/tkimage',
		'sk1sdk.libtk': 'src/libtk',
		'sk1sdk.libttk': 'src/libttk',
		'sk1sdk.tkpng': 'src/tkpng',
		'sk1sdk.tkstyle': 'src/tkstyle',
		'sk1sdk.tkXcursor': 'src/tkXcursor',
		},
			
		package_data = {
		'sk1sdk.tkpng': ['pkgIndex.tcl',],
		'sk1sdk': share_dirs,
		},
		
		ext_modules = [tkimage_module, paxtkinter_module, tkpng_module, tkXcursor_module])
	
#################################################
# .py source compiling
#################################################
if sys.argv[1]=='build':
	import compileall
	compileall.compile_dir('build/')
			
##############################################
# This section for developing purpose only
# Command 'python setup.py build&copy' allows
# automating build and native extension copying
# into package directory
##############################################
if COPY:
	import shutil, string, platform
	version=(string.split(sys.version)[0])[0:3]
	
	shutil.copy('build/lib.linux-'+platform.machine()+'-'+version+'/sk1sdk/tkimage/_imagingtk.so','src/tkimage/')
	print '\n _imagingtk.so has been copied to src/ directory'
	
	shutil.copy('build/lib.linux-'+platform.machine()+'-'+version+'/sk1sdk/libtk/_tkinter.so','src/libtk/')
	print '\n _tkinter.so has been copied to src/ directory'
	
	shutil.copy('build/lib.linux-'+platform.machine()+'-'+version+'/sk1sdk/tkpng/libtkpng.so','src/tkpng/')
	print '\n libtkpng.so has been copied to src/ directory'
			
	shutil.copy('build/lib.linux-'+platform.machine()+'-'+version+'/sk1sdk/tkXcursor/_tkXcursor.so','src/tkXcursor/')
	print '\n _tkXcursor.so has been copied to src/ directory'	
	
	os.system('rm -rf build')	
			
#################################################
# Implementation of bdist_deb command
#################################################
if DEBIAN:
	print '\nDEBIAN PACKAGE BUILD'
	print '===================='
	import shutil, string, platform
	version=(string.split(sys.version)[0])[0:3]
	
	arch,bin = platform.architecture()
	if arch=='64bit':
		arch='amd64'
	else:
		arch='i386'
		
	target='build/deb-root/usr/lib/python'+version+'/dist-packages'
	
	if os.path.lexists(os.path.join('build','deb-root')):
		os.system('rm -rf build/deb-root')
	os.makedirs(os.path.join('build','deb-root','DEBIAN'))
	
	os.system("cat DEBIAN/control |sed 's/<PLATFORM>/"+arch+"/g'|sed 's/<VERSION>/"+VERSION+"/g'> build/deb-root/DEBIAN/control")

	os.makedirs(target)	
	os.system('cp -R build/lib.linux-'+platform.machine()+'-'+version+'/sk1sdk '+target)	
		
	if os.path.lexists('dist'):	
		os.system('rm -rf dist/*.deb')
	else:
		os.makedirs('dist')
	
	os.system('dpkg --build build/deb-root/ dist/python-sk1sdk-'+VERSION+'_'+arch+'.deb')			
			
			
			
			
			
			
			
			
			
			
			
